#summarize the amplicon genotype calls into haplotype counts 

library(gplots)
library(gridExtra)
library(ggplot2)
library(egg)
library(dplyr)

gt = read.table("Hcor_ampliconSeq_plates1-12_shortRef_gatk_v2+bcftools_genotypes_manualEdited_AK.tsv", sep="\t", stringsAsFactors = FALSE)
colnames(gt)=gt[2,]
gt=gt[-1:-4,]
rownames(gt)=gt[,1]
gt=gt[,-1]

#subset genotypes to only include those indicated as yes in the last column
gt = gt[gt$.=="yes",1:11]
gt = gt[grepl("_20|Plate9|Plate10|Plate11|Plate12",rownames(gt)),] #double check; shouldn't remove any rows

write.table(gt,'Hcor_ampliconSeq_plates1-12_shortRef_gatk_v2+bcftools_genotypes_manualEdited_final.txt',
            quote = FALSE, row.names = TRUE, sep='\t', col.names=TRUE)

i=1
gt[gt[,i]=="T/T",i]=0
gt[gt[,i]=="A/T",i]=1
gt[gt[,i]=="A/A",i]=2

i=2
gt[gt[,i]=="T/T",i]=0
gt[gt[,i]=="G/T",i]=1
gt[gt[,i]=="G/G",i]=2

i=3
gt[gt[,i]=="C/C",i]=0
gt[gt[,i]=="C/T",i]=1
gt[gt[,i]=="T/T",i]=2

i=4
gt[gt[,i]=="C/C",i]=0
gt[gt[,i]=="C/T",i]=1
gt[gt[,i]=="T/T",i]=2

i=5
gt[gt[,i]=="C/C",i]=0
gt[gt[,i]=="C/A",i]=1
gt[gt[,i]=="A/A",i]=2

i=6
gt[gt[,i]=="A/A",i]=0
gt[gt[,i]=="A/T",i]=1
gt[gt[,i]=="T/T",i]=2

i=7
gt[gt[,i]=="C/C",i]=0
gt[gt[,i]=="C/T",i]=1
gt[gt[,i]=="T/T",i]=2

i=8
gt[gt[,i]=="C/C",i]=0
gt[gt[,i]=="C/T",i]=1
gt[gt[,i]=="T/T",i]=2

i=9
gt[gt[,i]=="T/T",i]=0
gt[gt[,i]=="A/T",i]=1
gt[gt[,i]=="A/A",i]=2

i=10
gt[gt[,i]=="T/T",i]=0
gt[gt[,i]=="A/T",i]=1
gt[gt[,i]=="A/A",i]=2

i=11
gt[gt[,i]=="A/A",i]=0
gt[gt[,i]=="A/G",i]=1
gt[gt[,i]=="G/G",i]=2

red=gt[endsWith(rownames(gt),"R"), ]
red_missing=colSums(red=="./.")
red[red=="./."] = 1 #assume missing genotypes are the most frequent class

green=gt[endsWith(rownames(gt),"G"), ]
green_missing=colSums(green=="./.")
green[green=="./."] =  0 #assume missing genotypes are the most frequent class

#compress the genotypes into one haplotype per sample
red$haplotype = apply(red, 1, function(x) paste(x[1:11], collapse = ""))
red_tally = red %>% group_by(haplotype) %>% tally %>% arrange(desc(n))
green$haplotype = apply(green, 1, function(x) paste(x[1:11], collapse = ""))
green_tally = green %>% group_by(haplotype) %>% tally %>% arrange(desc(n))

red_heatmap=matrix(,nrow=dim(red_tally)[1], ncol=11)
for (i in 1:dim(red_tally)[1]){
  red_heatmap[i,] = strsplit(red_tally$haplotype,split='')[[i]]
}
class(red_heatmap)="numeric"
rownames(red_heatmap)=paste0("Red Genotype ",1:dim(red_heatmap)[1])
colnames(red_heatmap)=colnames(gt)

green_heatmap=matrix(,nrow=dim(green_tally)[1], ncol=11)
for (i in 1:dim(green_tally)[1]){
  green_heatmap[i,] = strsplit(green_tally$haplotype,split='')[[i]]
}
class(green_heatmap)="numeric"
rownames(green_heatmap)=paste0("Green Genotype ",1:dim(green_heatmap)[1])
colnames(green_heatmap)=colnames(gt)

heatmap_all = rbind(green_heatmap,red_heatmap)
my_palette <- colorRampPalette(c("#56B4E9", "#E69F00", "#CC79A7"))(n = 299)
pdf("Hcor_ampliconSeq_plates1-12_shortRef_gatk_v2+bcftools_genotypes_All_heatmaps.pdf", width=7, height=7)
heatmap.2(heatmap_all, Rowv="NA", Colv="Rowv", labCol = FALSE, labRow = FALSE,trace="none", col=my_palette, density.info="none",
          dendrogram="none", key=FALSE, offsetRow=0.1, lwid=c(0.1,3), lhei=c(0.1,3),margin=c(3,3),
          sepwidth=c(0.05,0.05),sepcolor="white",colsep=1:ncol(heatmap_all),rowsep=1:nrow(heatmap_all),
          RowSideColors=c(rep("#009E73",dim(green_heatmap)[1]),rep("#D55E00",dim(red_heatmap)[1])),
          cexRow=1.1,cexCol=1.7)
dev.off()

missing=rbind(green_missing,red_missing)
write.table(missing,'Hcor_ampliconSeq_plates1-12_shortRef_gatk_v2+bcftools_genotypes_missing.txt',
            quote = FALSE, row.names = TRUE, sep='\t', col.names=TRUE)

#make bar plots of the genotype tallies
red_tally$names = rownames(red_heatmap)
red_tally$names = factor(red_tally$names,levels = red_tally$names)
red_plot =red_tally %>% ggplot(aes(x=reorder(names, n), y=n/sum(n))) +
  geom_bar(stat="identity", fill="#D55E00") + coord_flip() + 
    labs(x = "", y = "Proportion of samples") +
    geom_text(aes(label = n), hjust = -0.2) +
    theme(axis.line.x = element_line(size = .6, colour = "black"),
          axis.text = element_text(size = 11), 
          axis.title = element_text(size = 11), 
          axis.title.y=element_blank(),
          axis.text.y=element_blank(),
          axis.ticks.y=element_blank(),
          panel.background = element_blank())  + ylim(0,1)

green_tally$names = rownames(green_heatmap)
green_tally$names = factor(green_tally$names,levels = green_tally$names)
green_plot = green_tally %>% ggplot(aes(x=reorder(names, n), y=n/sum(n))) +
  geom_bar(stat="identity", fill="#009E73") + coord_flip() + 
  labs(x = "", y = "") +
  geom_text(aes(label = n), hjust = -0.2) +
  theme(axis.title.x=element_blank(),
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank(),
        axis.title.y=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank(),
        panel.background = element_blank())  + ylim(0,1)

#need to plot both histograms together
library(forcats)

red_tally$prop = red_tally$n/sum(red_tally$n)
green_tally$prop = green_tally$n/sum(green_tally$n)
plot_tally=rbind(green_tally,red_tally)
tally = plot_tally %>% ggplot(aes(x=fct_rev(names), y=prop)) +
  geom_bar(stat="identity", fill=c(rep("#009E73",5), rep("#D55E00",17))) + coord_flip() + 
  labs(x = "", y = "Proportion of samples per gall color", title = "Number of samples") +
  geom_text(aes(label = n), hjust = -0.30, size=5.5) +
  theme(plot.title = element_text(size=18, hjust = 0.5),
        axis.line.x = element_line(size = .6, colour = "black"),
        axis.text = element_text(size = 18), 
        axis.title = element_text(size = 18), 
        axis.title.y=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank(),
        panel.background = element_blank())  + ylim(0,1.05) 

pdf("Hcor_ampliconSeq_plates1-12_shortRef_gatk_v2+bcftools_genotypes_histograms.pdf", width=5, height=7)
tally
dev.off()

#############################################################################################################
#LOD calculation using the amplicon seq data, taking proper account of the missing sites
#the C and D variables are match class and mismatch class, not the two classes of phenotypes
#the 0.5 ^ samples size used in the denominator is because it's a binary variable with half the chance of geno and 
#pheno matching under the null
#this calculation lumps red 0/1 and 1/1

calculateLOD <- function(C,D) {
  N=C+D
  theta=C/N
  LOD=log(((theta^C)*(1-theta)^D)/0.5^(C+D),10)
  return(LOD)
}

red=gt[endsWith(rownames(gt),"R"), ]
green=gt[endsWith(rownames(gt),"G"), ]

LOD=vector(mode="numeric",11)
for(i in 1:11){
  match=sum(red[,i]==1 | red[,i]==2)+sum(green[,i]==0)
  mismatch=sum(red[,i]==0)+sum(green[,i]==1 | green[,i]==2)
  LOD[i]=calculateLOD(match,mismatch)
}

#plot the LOD numbers as the row names of the genotype heatmap
colnames(heatmap_all) = round(LOD,0)
my_palette <- colorRampPalette(c("#56B4E9", "#E69F00", "#CC79A7"))(n = 299)
pdf("Hcor_ampliconSeq_plates1-12_shortRef_gatk_v2+bcftools_genotypes_All_heatmaps.pdf", width=7, height=7)
heatmap.2(heatmap_all, Rowv="NA", Colv="Rowv", labRow = FALSE, trace="none", col=my_palette, density.info="none",
          dendrogram="none", key=FALSE, offsetRow=0.1, lwid=c(0.1,3), lhei=c(0.1,3),margin=c(4,3),
          sepwidth=c(0.05,0.05),sepcolor="white",colsep=1:ncol(heatmap_all),rowsep=1:nrow(heatmap_all),
          RowSideColors=c(rep("#009E73",dim(green_heatmap)[1]),rep("#D55E00",dim(red_heatmap)[1])),
          cexRow=1.1,cexCol=2.3, srtCol = 0, adjCol = c(0.5,1)) 
dev.off()

